package visitor;

import visitor.contexts.WaterPumperAgent;

// Implementação do Visitor de interpretação do workflow
public class InterpretingVisitor implements ActionVisitor {

    // O contexto do Visitor é definido nos atributos
    private WaterPumperAgent context;

    public InterpretingVisitor(WaterPumperAgent context) {
        this.context = context;
    }

    // Para uma ação, um elemento terminal, a ação é executada
    public void visit(CommandAction node) {
        node.getCommand().execute(this.context);
    }

    // Para uma sequência da ações, a operação que chama o visitor é chamad apara cada uma delas
    public void visit(SequenceAction node) {
        node.getFirstAction().accept(this);
        node.getSecondAction().accept(this);
    }

    // Para uma alternação entre ações, uma ou outra ação é chamada, dependendo do resultado da condição
    public void visit(AlternationAction node) {
        if(node.getCondition().evaluate(this.context))
            node.getAction().accept(this);
        else {
            Action falseAction = node.getFalseAction();
            if(falseAction != null) falseAction.accept(this);
        }
    }

    // Para um laço enquanto-faça, enquanto a condição for verdadeira a ação é chamada
    public void visit(WhileRepetitionAction node){
        while (node.getCondition().evaluate(this.context)){
            node.getAction().accept(this);
        }
    }

    // Para um laço faça-enquanto, enquanto a condição for falsa a ação é chamada
    public void visit(UntilRepetitionAction node){
        do {
            node.getAction().accept(this);
        } while (!node.getCondition().evaluate(this.context));
    }

}
