package visitor;

public class XMLGeneratingVisitor implements ActionVisitor {

    // Para que a string seja acumulada, um atributo XMLString é definido
    private String XMLString="";

    public String getXMLString() {
        return XMLString;
    }

    // Gera o XML para a ação terminal
    public void visit(CommandAction node) {

        XMLString = XMLString.concat("<CommandAction>");

        String commandName = node.getCommand().getClass().getSimpleName();

        XMLString = XMLString.concat(
                "<" + commandName + ">" +
                        node.getCommand().getValue().toString() +
                        "</" + commandName + ">");

        XMLString = XMLString.concat("</CommandAction>");
    }

    // Gera o XML para as duas ações
    public void visit(SequenceAction node) {

        XMLString = XMLString.concat("<SequenceAction>");

        node.getFirstAction().accept(this);
        node.getSecondAction().accept(this);

        XMLString = XMLString.concat("</SequenceAction>");
    }

    // Gera o XML para a condição, a ação caso verdadeira e a ação caso falsa
    public void visit(AlternationAction node) {

        XMLString = XMLString.concat("<AlternationAction>");

        String conditionName = node.getCondition().getClass().getSimpleName();

        XMLString = XMLString.concat(
                "<" + conditionName + ">" +
                        node.getCondition().getParameter().toString() +
                        "</" + conditionName + ">");

        node.getAction().accept(this);

        if(node.getFalseAction() != null) {
            XMLString = XMLString.concat("<Else>");
            node.getFalseAction().accept(this);
            XMLString = XMLString.concat("</Else>");
        }

        XMLString= XMLString.concat("</AlternationAction>");
    }

    // Gera o XML para a condição e para a ação
    public void visit(WhileRepetitionAction node){
        XMLString = XMLString.concat("<WhileRepetitionAction>");

        String conditionName = node.getCondition().getClass().getSimpleName();

        XMLString = XMLString.concat(
                "<" + conditionName + ">" +
                        node.getCondition().getParameter().toString() +
                        "</" + conditionName + ">");

        node.getAction().accept(this);

        XMLString = XMLString.concat("</WhileRepetitionAction>");
    }

    // Gera o XML para a condição e para a ação
    public void visit(UntilRepetitionAction node){
        XMLString = XMLString.concat("<UntilRepetitionAction>");

        String conditionName = node.getCondition().getClass().getSimpleName();

        XMLString = XMLString.concat(
                "<" + conditionName + ">" +
                        node.getCondition().getParameter().toString() +
                        "</" + conditionName + ">");

        node.getAction().accept(this);

        XMLString = XMLString.concat("</UntilRepetitionAction>");
    }
}
